<?php
/**
 * Plugin main class.
 *
 * @package WPDesk\FlexibleCoupons
 */

namespace WPDesk\FlexibleCoupons;

use FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Integration\SampleTemplates;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use FlexibleCouponsVendor\WPDesk\View\Renderer\Renderer;
use FlexibleCouponsVendor\WPDesk\Logger\WPDeskLoggerFactory;
use FlexibleCouponsVendor\WPDesk_Plugin_Info;
use FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\AbstractPlugin;
use FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\HookableCollection;
use FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\HookableParent;
use FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\Activateable;
use Psr\Log\NullLogger;
use WPDesk\FlexibleCoupons\Email\RegisterEmails;
use WPDesk\FlexibleCoupons\ReviewNotice\ReviewNotice;
use WPDesk\FlexibleCoupons\ReviewNotice\TwoWeeksNotice;
use WPDesk\FlexibleCoupons\Settings\GeneralSettings;
use WPDesk\FlexibleCoupons\Tracker\Tracker;

/**
 * Main plugin class. The most important flow decisions are made here.
 *
 * @package WPDesk\FlexibleCoupons
 */
class Plugin extends AbstractPlugin implements LoggerAwareInterface, HookableCollection, Activateable {
	use LoggerAwareTrait;
	use HookableParent;

	/**
	 * @var string
	 */
	private $plugin_path;

	/**
	 * @var Renderer
	 */
	private $renderer;

	/**
	 * @var string
	 */
	private $scripts_version = '1.0';

	const EDITOR_POST_TYPE = 'wpdesk-coupons';

	/**
	 * Plugin constructor.
	 *
	 * @param WPDesk_Plugin_Info $plugin_info Plugin info.
	 */
	public function __construct( WPDesk_Plugin_Info $plugin_info ) {
		parent::__construct( $plugin_info );

		$this->plugin_info      = $plugin_info;
		$this->plugin_url       = $this->plugin_info->get_plugin_url();
		$this->plugin_path      = $this->plugin_info->get_plugin_dir();
		$this->plugin_namespace = $this->plugin_info->get_text_domain();

		$is_pl              = 'pl_PL' === get_locale();
		$this->settings_url = admin_url( 'edit.php?post_type=' . self::EDITOR_POST_TYPE . '&page=fc-settings' );
		$this->docs_url     = $is_pl ? 'https://www.wpdesk.pl/docs/flexible-coupons-pro/' : 'https://www.wpdesk.net/docs/flexible-coupons-pro/';
		$this->support_url  = $is_pl ? 'https://www.wpdesk.pl/support/' : 'https://www.wpdesk.net/support/';
	}

	/**
	 * Initializes plugin external state.
	 * The plugin internal state is initialized in the constructor and the plugin should be internally consistent after
	 * creation. The external state includes hooks execution, communication with other plugins, integration with WC
	 * etc.
	 *
	 * @return void
	 */
	public function init() {
		$this->setLogger( $this->is_debug_mode() ? ( new WPDeskLoggerFactory() )->createWPDeskLogger() : new NullLogger() );
		$this->hooks();
	}

	/**
	 * Returns true when debug mode is on.
	 *
	 * @return bool
	 */
	private function is_debug_mode(): bool {
		$helper_options = get_option( 'wpdesk_helper_options', [] );

		return isset( $helper_options['debug_log'] ) && '1' === $helper_options['debug_log'];
	}

	/**
	 * @return void
	 */
	public function hooks() {
		parent::hooks();

		$editor = new RegisterEditor( self::EDITOR_POST_TYPE );
		$coupon = new CouponIntegration( $editor, $this->logger );
		$this->add_hookable( $editor );
		$this->add_hookable( $coupon );
		$this->add_hookable( new GeneralSettings() );
		$this->add_hookable( new RegisterEmails( $this->plugin_info ) );
		$this->add_hookable( new Tracker() );

		add_action( 'woocommerce_init', function () {
			if ( is_admin() ) {
				( new ReviewNotice(
					[ new TwoWeeksNotice( $this->plugin_url . '/assets' ) ]
				) )->hooks();
			}
		} );

		$this->hooks_on_hookable_objects();
	}

	/**
	 * Plugin activation.
	 */
	public function activate() {
		$post = new SampleTemplates( self::EDITOR_POST_TYPE );
		$post->create();
	}

}
