<?php

/**
 * Integration. Order.
 *
 * @package WPDesk\Library\WPCoupons
 */
namespace FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Order;

use FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Integration\Helper;
use FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Integration\PostMeta;
use FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Settings\Settings;
use FlexibleCouponsVendor\WPDesk\Persistence\Adapter\WordPress\WordpressOptionsContainer;
use FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use FlexibleCouponsVendor\WPDesk\View\Renderer\Renderer;
use WC_Coupon;
use WC_Order;
use WC_Order_Item;
use WC_Order_Item_Product;
use WP_Post;
/**
 * Add coupon meta box to order post type.
 *
 * @package WPDesk\Library\WPCoupons\Integration
 */
class OrderMetaBox implements \FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\Hookable
{
    const META_BOX_CONTEXT_SIDE = 'side';
    const META_BOX_PRIORITY_LOW = 'low';
    const PRODUCT_TYPE = 'wpdesk_pdf_coupons';
    /**
     * @var Renderer
     */
    private $renderer;
    /**
     * @var PostMeta
     */
    private $postmeta;
    /**
     * @param Renderer $renderer
     * @param PostMeta $postmeta
     */
    public function __construct(\FlexibleCouponsVendor\WPDesk\View\Renderer\Renderer $renderer, \FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Integration\PostMeta $postmeta)
    {
        $this->renderer = $renderer;
        $this->postmeta = $postmeta;
    }
    /**
     * Fires hooks.
     */
    public function hooks()
    {
        \add_action('add_meta_boxes', array($this, 'register_meta_boxes_action'));
    }
    /**
     * Add custom meta box to order page.
     *
     * @return void|bool
     */
    public function register_meta_boxes_action()
    {
        global $post;
        if (!isset($post)) {
            return \false;
        }
        if (isset($post->post_type) && $post->post_type !== 'shop_order') {
            return \false;
        }
        /**
         * @var $order WC_Order Order.
         */
        $order = \wc_get_order($post->ID);
        if ($order instanceof \WC_Order) {
            $items = $order->get_items();
            foreach ($items as $item) {
                if ($item instanceof \WC_Order_Item_Product) {
                    $product_id = $item->get_product_id();
                    $is_coupon_item = 'yes' === $this->postmeta->get_private($product_id, self::PRODUCT_TYPE);
                    if ($is_coupon_item) {
                        \add_meta_box('flexible_coupon__' . $item->get_product_id() . '_' . $item->get_id(), \__('PDF Coupon', 'flexible-coupons'), array($this, 'order_coupon_callback'), 'shop_order', self::META_BOX_CONTEXT_SIDE, self::META_BOX_PRIORITY_LOW, array('post_id' => $post->ID, 'item' => $item, 'item_id' => $item->get_id()));
                    }
                }
            }
        }
    }
    /**
     * @param WP_Post       $post Post.
     * @param array $item Order item.
     */
    public function order_coupon_callback(\WP_Post $post, $item)
    {
        /**
         * @var $item WC_Order_Item_Product
         */
        $item = $item['args']['item'];
        $meta_coupon_name = 'fcpdf_order_item_' . $item->get_id() . '_coupon_id';
        $coupon_id = (int) $this->postmeta->get_private($post->ID, $meta_coupon_name);
        $coupon = new \WC_Coupon($coupon_id);
        $coupon_data = $this->postmeta->get_private($coupon_id, 'fcpdf_coupon_data', []);
        $coupon_code = $coupon->get_id() ? $coupon->get_code() : '';
        $coupon_url = $coupon->get_id() ? \admin_url('post.php?post=' . $coupon_id . '&action=edit') : '#';
        $product = \wc_get_product($item->get_product_id());
        $product_id = $product->get_id();
        if ($product->is_type('variation')) {
            $product_id = $product->get_parent_id();
        }
        $product_url = $coupon->get_id() ? \admin_url('post.php?post=' . $product_id . '&action=edit') : '#';
        $download_url = $coupon->get_id() ? \FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Integration\Helper::make_coupon_url($coupon_data) : '';
        $data = array('order_id' => $post->ID, 'item_id' => $item->get_id(), 'product_id' => $item->get_product_id(), 'variation_id' => $item->get_variation_id(), 'product_name' => $item->get_name(), 'product_url' => $product_url, 'coupon_id' => $coupon->get_id(), 'coupon_url' => $coupon_url, 'coupon_code' => $coupon_code, 'coupon_is_used' => $this->is_coupon_limit_reached($coupon), 'coupon_title' => $coupon_code, 'download_url' => $download_url, 'quantity' => $item->get_quantity());
        $this->renderer->output_render('html-order-coupon', $data);
        //phpcs:ignore
    }
    /**
     * @param WC_Coupon $coupon Coupon data.
     *
     * @return bool
     */
    private function is_coupon_limit_reached(\WC_Coupon $coupon) : bool
    {
        return $coupon->get_usage_limit() > 0 && $coupon->get_usage_count() >= $coupon->get_usage_limit();
    }
}
